#!/bin/bash
name=$(basename ${0})

wah_error() {
	>&2 echo "$1"
	exit 1
}

# This script assumes that the dhclient startup command from the inet.defn file
# of the ifupdown package has the following format:
# dhclient -4 -v -pf /run/dhclient.%iface%.pid \
#       -lf /var/lib/dhcp/dhclient.%iface%.leases -I \
#       -df /var/lib/dhcp/dhclient6.%iface%.leases %iface%
wah_restart_dhclient() {
	local iface=$1
	local dhclient_pidfile="/run/dhclient.$iface.pid"

	# Check if there's a PID file for iface and its owner is root.
	[[ `stat -c %U "$dhclient_pidfile"` == "root" ]] || return 1

	local dhclient_pid=`cat "$dhclient_pidfile"`
	local dhclient_cmdline=`cat "/proc/$dhclient_pid/cmdline" | tr '\0' ' '`

	# We restart dhclient using the same cmdline it was exec'ed with,
	# so we have validate it.

	if [[ -z $dhclient_cmdline ]] \
		|| [[ ! $dhclient_cmdline =~ ^dhclient ]]; then
		return 1
	fi

	logger -p local1.error -t "${name}" "restarting dhclient..."
	dhclient -r -pf "$dhclient_pidfile" "$iface"
	$dhclient_cmdline
}

wah_restart_dhcpcd() {
	local iface=$1
	local dhcpcd_pidfile="/run/dhcpcd/${iface}.pid"
	local dhcpcd_pid=$(< "${dhcpcd_pidfile}")

	# Check if there's a PID file for iface and its owner is root.
	[[ $(stat -c %U "$dhcpcd_pidfile") == "root" ]] || return 1

	logger -p local1.error -t "${name}" "rebinding dhcpcd..."

	# Notifies dhcpcd to reload its configuration and rebind the specified interface.
	dhcpcd --rebind ${iface} |& logger  -p local1.error -t "${name}"
}

[[ $# -eq 2 ]] || wah_error "usage: $0 IFACE ACTION"

IFACE=$1
ACTION=$2

logger -p local1.info -t "${name}" "${name}: ACTION=${ACTION}"

if [[ $ACTION == "CONNECTED" ]]; then
	if command -v dhcpcd >& /dev/null; then
		wah_restart_dhcpcd "$IFACE"
	elif command -v dhclient >& /dev/null; then
		wah_restart_dhclient "$IFACE"
	fi
fi

exit 0
